#!/bin/ksh
#
###############################################################################
# SHELL SCRIPT SOURCE FILE
#
# PROGRAM NAME :       HealthCheck.sh
# DESCRIPTION :        This script has been developed to monitor and report the
#                      UNIX components. This script builds on an AIX script
#												written by a Wily client.
#
###############################################################################
#
###############################################################################
# CHANGE DESCRIPTION :    Draft version with FIX option
# 16/06/2004 - John MillerChip - V1.3 JohnM Catch any errors and don't pass onto Wily scripts
# 28/06/2007 - Parag Mungale/Sufyaan Kazi (CA Wily)   - Removed redundant methods, added method to track EM Process
# 03/07/2007 - Sufyaan Kazi (CA Wily) - Tideid up and add separate file for getting strng arguments
# 03/07/2007 - Sufyaan Kazi (CA Wily) - Added function to monitor the network interfaces, commentd out
#                                       functions to monitor io and disks
# 10/07/2007 - Sufyaan Kazi (CA Wily) - Uncommented network and disk io sections
# 04/09/2007 - Sufyaan Kazi (CA Wily) - Modified script to be UNIX only, eliminated Perl
#	19/09/2007 - Sufyaan Kazi (CA Wily) - Removed hardcoding for Ethernet port
# 27/09/2007 - Sufyaan Kazi (CA Wily) - Tidied up script
#
#############################################################################################

# Global variables

HOSTNAME=`hostname`
DATE=`date +%d-%m-%Y:%H:%m`

#Folder names to appear under HealthCheck in the Investigator Tree
MEM="Memory"
MISC="Misc"
NETW="NetworkIO"
DIO="DiskIO"
FS="Filesystem"
ISCOPE="Introscope"
THR="KernelThreads"
PAGE="Paging"
CPU="CPU"
FLTS="Faults"


# Function to write out a long metric in XML format for the EPA
# Arg 1 - Metric Name and Value (e.g. User=40)
# Arg 2 - Sub-Folder to stitch the metric in the Investigator Tree under HealthCheck
long_metric()
{
        write_metric "$1" "$2" "LongCounter"
}

#Function to write a metric in XML format for the EPA
# Arg 1 - Metric Name and Value (e.g. User=40)
# Arg 2 - Sub-Folder to stitch the metric in the Investigator Tree under HealthCheck
# Arg 3 - Metric Type, e.g. LongCounter
write_metric()
{
        typeset -i number=0

        #Get metrics and value
        metric=`print $1 | cut -f1 -d "="`
        number=`print $1 | cut -f2 -d "="`
        
        #Get folder to put metrics in
        if [[ -z "$2" ]]
        then
        	folder="Metrics"
        else
        	folder=$2
        fi
        
        #Get Metric Types
        metricType=$3

        print "<metric type=\"${metricType}\"  name=\"HealthCheck|${folder}:${metric}\" value=\"${number}\"/>"
}

#Function to get the total Paging Space and %Used
PAGING_status()
{
        typeset -i PAGING_TOT=0
        typeset -i PAGING_USED=0
        PAGING_TOT=`lsps -s | tail -1 | cut -d" " -f7| cut -d"MB" -f1`
        PAGING_USED=`lsps -s | tail -1 | cut -d" " -f21| cut -d"%" -f1`

        long_metric "PAGING_TOT=$PAGING_TOT"  $PAGE
        long_metric "PAGING_USED=$PAGING_USED" $PAGE
}

#Function to count the no. of processes and zombies
NUM_procs()
{
        #No. of running processes
        NUM_PROCS=`ps -efa | wc -l`
        NUM_PROCS=`expr $NUM_PROCS`


        #Of which, no. of zombies
        NUM_ZOMBIES=`ps -efa | awk '{ if ( $1 != "root" && $2 == "1" && $3 == "1" ){ print $0 } }' | wc -l`
        NUM_ZOMBIES=`expr $NUM_ZOMBIES`

        long_metric "NUM_PROCS=$NUM_PROCS" $MISC
        long_metric "NUM_ZOMBIES=$NUM_ZOMBIES" $MISC
}

#Function to view Filesystem used values
FILESYS_status()
{
        for FILESYS in `mount | egrep -v "nfs|procfs|cdrfs" | grep -iv mount | awk '{ if (NR>1){ print $2 }}'`
        do
           long_metric `df -k ${FILESYS} | grep -iv mount | awk '{ printf ("%s%s=%s\n","FILESYS_",$7,$4) }' 2>/dev/null | cut -d"%" -f1` $FS
        done
}

#Function to get status of Network ports
ETHERNET_status()
{
        for i in `netstat -i | grep -i en | cut -d" " -f1 | sort | uniq`
        do
                typeset -i DEV_I=1
                DEV=`entstat -d $i | grep -i "Link Status" | cut -d" " -f3`
                if [[ $DEV != "Up" ]]
                then
                        DEV_I=0
                fi

                long_metric "ETHERNET_${i}_STATUS=$DEV_I" $NETW
        done
}

#Function to get size of packets and errors
NETWORK_loads()
{
        for NI in `netstat -i | grep -i en | cut -d" " -f1 | sort | uniq`
        do
                NIC=`netstat -i | grep -i ${NI} | grep -iv link | awk '{ printf ("%s,%s,%s,%s,%s,%s,%s,%s,%s\n", $1,$2,$3,$4,$5,$6,$7,$8,$9) }'`
                IPkts=`print $NIC | cut -f5 -d "," `
                Ierrs=`print $NIC | cut -f6 -d "," `
                OPkts=`print $NIC | cut -f7 -d "," `
                Oerrs=`print $NIC | cut -f8 -d "," `
                long_metric "NIC_${NI}_Incoming_Data_Packets=${IPkts}" $NETW
                long_metric "NIC_${NI}_Incoming_Data_Errors=${Ierrs}" $NETW
                long_metric "NIC_${NI}_Transmitted_Data_Packets=${OPkts}" $NETW
                long_metric "NIC_${NI}_Transmitted_Data_Errors=${Oerrs}" $NETW
        done
}

#Function to get IO on disks
DRIVEIO_loads()
{
        #Get the list & count of Drives
        iostat -d | grep -i "hdisk" |awk '{  print $1 }' > /tmp/drive_list
        DriveCount=`wc -l /tmp/drive_list | awk '{ printf ("%s", $1) }'`

        #Get the details for all adapters
        iostat -d 1 2 | grep -i "hdisk" | tail -n ${DriveCount} |  awk '{ printf ("%s,%s,%s,%s,%s,%s\n", $1,$2,$3,$4,$5,$6) }' > /tmp/driveOutput

        #Increment the count to get the loop right
        DriveCount=`expr $DriveCount + 1`

        #Extract data needed for each adapter
        count=1
        while [ $count -lt $DriveCount ]
        do
             #Pop a line from the file
             DISKIO=`tail -n ${count} /tmp/driveOutput | head -n 1`

             #Read the line in
             adapter=`print $DISKIO | cut -f1 -d "," `
             tm_act=`print $DISKIO | cut -f2 -d "," `
             KBPS=`print $DISKIO | cut -f3 -d "," `
             tps=`print $DISKIO | cut -f4 -d "," `
             kb_read=`print $DISKIO | cut -f5 -d "," `
             kb_wrtn=`print $DISKIO | cut -f6 -d "," `
             long_metric "DISK_${adapter}_%_Time_Disk_was_Active=${tm_act}" $DIO
             long_metric "DISK_${adapter}_KBPS=${KBPS}" $DIO
             long_metric "DISK_${adapter}_tps=${tps}" $DIO
             long_metric "DISK_${adapter}_Kb_read=${kb_read}" $DIO
             long_metric "DISK_${adapter}_Kb_wrtn=${kb_wrtn}" $DIO

             #Increment the counter
             count=`expr $count + 1`
        done
}

#Function to get VMStat Metrics
VMSTAT()
{
        typeset -i RUNQ=0       #Threads in the Runnable Queue
        typeset -i WAITQ=0      #Threads in the Wait Queue
        typeset -i AVM=0        #Active Virtual Pages
        typeset -i FRE=0        #Size of the Free List
        typeset -i RE=0         #Paging Input/Output List
        typeset -i PI=0         #Pages paged IN from paging space
        typeset -i PO=0         #Pages paged OUT from paging space
        typeset -i FR=0         #Pages freed
        typeset -i SR=0         #Pages scanned by the paging replacement algorithm
        typeset -i CY=0         #Clock Cycles by Page Replacement Algorithm
        typeset -i IN=0         #Device Interrupts
        typeset -i SI=0         #System Interrupts
        typeset -i CS=0         #Kernel Thread Context Switches
        typeset -i US=0         #% CPU usage broke down to User Time
        typeset -i SY=0         #% CPU usage broke down to System Time
        typeset -i ID=0         #% CPU usage broke down to Idle Time
        typeset -i WA=0         #% CPU usage broke down to Idle Time which the system was waiting on IO

        VM=`vmstat 1 3 | tail -1 | awk '{ printf ("%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n", $1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17) }'`

        RUNQ=`print $VM | cut -f1 -d "," `
        WAITQ=`print $VM | cut -f2 -d "," `
        AVM=`print $VM | cut -f3 -d "," `
        FRE=`print $VM | cut -f4 -d "," `
        RE=`print $VM | cut -f5 -d "," `
        PI=`print $VM | cut -f6 -d "," `
        PO=`print $VM | cut -f7 -d "," `
        FR=`print $VM | cut -f8 -d "," `
        SR=`print $VM | cut -f9 -d "," `
        CY=`print $VM | cut -f10 -d "," `
        IN=`print $VM | cut -f11 -d "," `
        SI=`print $VM | cut -f12 -d "," `
        CS=`print $VM | cut -f13 -d "," `
        US=`print $VM | cut -f14 -d "," `
        SY=`print $VM | cut -f15 -d "," `
        ID=`print $VM | cut -f16 -d "," `
        WA=`print $VM | cut -f17 -d "," `

        long_metric "Runnables Queue=${RUNQ}" $THR
        long_metric "Wait Queue=${WAITQ}" $THR
        long_metric "Active Virtual Pages=${AVM}" $MEM
        long_metric "Free List=${FRE}" $MEM
        long_metric "Pager Input/Output List=${RE}" $PAGE
        long_metric "Paged In=${PI}" $PAGE
        long_metric "Paged Out=${PO}" $PAGE
        long_metric "Pages Freed=${FR}" $PAGE
        long_metric "Pages Scanned=${SR}" $PAGE
        long_metric "Pager Clock Cycles=${CY}" $PAGE
        long_metric "Device Interrupts=${IN}" $FLTS
        long_metric "System Interrupts=${SI}" $FLTS
        long_metric "Kernel Thread Context Switches=${CS}" $FLTS
        long_metric "User=${US}" $CPU
        long_metric "System=${SY}" $CPU
        long_metric "Idle=${ID}" $CPU
        long_metric "Idle (w/outstanding IO)=${WA}" $CPU
}

#Function to view Error Reports for the last n hours
ERROR_status()
{
# TIME is being set here to look over the last hour, this period can be changed by simply changing the following
# line

        TIME=`TZ=GMT+1 date +%m%d%H%M%y`

        NO_ERRS=`errpt -s $TIME | grep -v IDENTIFIER | wc -l `
        NO_ERRS=`expr $NO_ERRS`
        long_metric "ERR_STATUS=$NO_ERRS" $MISC
}

#TOP5_CPU_procs()
#{
#        counter=0
#        for PID in `ps -eo user,pid,pcpu,args| grep -iv PID |sort -r -k 3,3 | head -n 5 | awk '{ printf ("%s\n",$2) }'`
#        do
#              counter=`expr $counter + 1`
#              PROCNAME=`ps -o args -p ${PID} | grep -iv COMMAND`
#              PROCVAL=`ps -o pcpu -p ${PID} | grep -iv CPU`
#              print "TOP5_CPU_PROCESS_${counter}=${PROCNAME} - ${PROCVAL}%"
#        done
#}

#Function to get Memory and CPU used of an EM (if relevant)
EM_PROC_status()
{
        typeset -i DURATION=5
        typeset -i SECONDS1=0
        typeset -i SECONDS2=0
        typeset -i CPU_DIFF=0
        typeset -i MEM=0
        typeset -i PCPU=0

        #################################
        #                               #
        #       MEMORY                  #
        #                               #
        #################################

        #Get Current Mem Usage
        MEM=`ps -eo user,pid,vsz,args| grep -iv USER | grep -iv grep |grep "Introscope_Enterprise_Manager" | awk '{ printf ("%s\n",$3) }'`
        MEM=$((MEM/1024))
        long_metric "EM_MEM_USAGE=${MEM}" $ISCOPE

        #################################
        #                               #
        #       CPU                  #
        #                               #
        #################################

        #Get current CPUTime
        CPUTIME1=`ps -eo user,pid,time,args| grep -iv USER | grep -iv grep |grep "Introscope_Enterprise_Manager" | awk '{ printf ("%s\n",$3) }'`
        SECONDS1=`convert_to_seconds ${CPUTIME1}`
        sleep $DURATION
        #Get new CPUTTIME
        CPUTIME2=`ps -eo user,pid,time,args| grep -iv USER | grep -iv grep |grep "Introscope_Enterprise_Manager" | awk '{ printf ("%s\n",$3) }'`
        SECONDS2=`convert_to_seconds ${CPUTIME2}`
        #Calculate %CPU
        CPU_DIFF=$(( SECONDS2 - SECONDS1 ))
        if (( $CPU_DIFF > $DURATION ))
        then
                DURATION=$CPU_DIFF
        else
                DURATION=$DURATION
        fi
        PCPU=$(( ((CPU_DIFF)*100)/DURATION))

        long_metric "EM_CPU_USAGE=${PCPU}" $ISCOPE
}

#Function to get Memory and CPU used of WebView (if relevant)
WebView_PROC_status()
{
        typeset -i DURATION=5
        typeset -i SECONDS1=0
        typeset -i SECONDS2=0
        typeset -i CPU_DIFF=0
        typeset -i MEM=0
        typeset -i PCPU=0

        #################################
        #                               #
        #       Memory                  #
        #                               #
        #################################

        #Get Current Mem Usage
        MEM=`ps -eo user,pid,vsz,args| grep -iv USER | grep -iv grep |grep "Introscope_WebView" | awk '{ printf ("%s\n",$3) }'`
        MEM=$((MEM/1024))
        long_metric "WebView_MEM_USAGE=${MEM}" $ISCOPE

        #################################
        #                               #
        #       CPU                  #
        #                               #
        #################################

        #Get current CPUTime
        CPUTIME1=`ps -eo user,pid,time,args| grep -iv USER | grep -iv grep |grep "Introscope_WebView" | awk '{ printf ("%s\n",$3) }'`
        SECONDS1=`convert_to_seconds ${CPUTIME1}`
        sleep $DURATION
        #Get new CPUTTIME
        CPUTIME2=`ps -eo user,pid,time,args| grep -iv USER | grep -iv grep |grep "Introscope_WebView" | awk '{ printf ("%s\n",$3) }'`
        SECONDS2=`convert_to_seconds ${CPUTIME2}`
        #Calculate %CPU
        CPU_DIFF=$(( SECONDS2 - SECONDS1 ))
        if (( $CPU_DIFF > $DURATION ))
        then
                DURATION=$CPU_DIFF
        else
                DURATION=$DURATION
        fi
        PCPU=$(( ((CPU_DIFF)*100)/DURATION))

        long_metric "WebView_CPU_USAGE=${PCPU}" $ISCOPE
}

#Utility function to convert time reported in ps to seconds
convert_to_seconds()
{
        #Grab the input time - format: [dd-]hh:mm:ss
        TIME=$1
        #print "Input Arg is: ${TIME}"

        typeset -i COUNT=0
        typeset -i DAYS=0
        typeset -i HOURS=0
        typeset -i MINUTES=0
        typeset -i SECONDS=0
        typeset -i TOTAL_SECONDS=0

        #Try to grab the days
        DAYS=`print $1 | cut -f1 -d "-" -s`
        #print "Days is: ${DAYS}"

        #print "Count is: ${COUNT}"
        if (( ${DAYS} != 0 ))
        then
                #print "There were days"
                TIME=`print $1 | cut -f2 -d "-" -s`
        else
                #print "There are no days"
                DAYS=0
        fi

        #Extract hours, minutes and seconds
        HOURS=`print $TIME | cut -f1 -d ":" -s`
        MINUTES=`print $TIME | cut -f2 -d ":" -s`
        SECONDS=`print $TIME | cut -f3 -d ":" -s`

        #print "Output is: ${DAYS} + ${HOURS} + ${MINUTES} + ${SECONDS}\n"

        #Compute values in Seconds
        DAYS=$((DAYS * 60 * 60 * 24))
        HOURS=$((HOURS * 60 * 60))
        MINUTES=$((MINUTES * 60))

        #print "Modified Output is: ${DAYS} + ${HOURS} + ${MINUTES} + ${SECONDS}\n"
        TOTAL_SECONDS=$((DAYS+HOURS+MINUTES+SECONDS))

        #Return the results
        print "${TOTAL_SECONDS}\n"
}

# Main script

NUM_procs
FILESYS_status
ETHERNET_status
ERROR_status
PAGING_status
DRIVEIO_loads
NETWORK_loads
VMSTAT

#Uncomment these if running on the same box as an Introscope Enterprise Manager and/or Webview
#EM_PROC_status
#WebView_PROC_status

#SK - Commented out, this will cause Metric Explosion
#TOP5_CPU_procs