#!/bin/ksh93
#
###############################################################################
# SHELL SCRIPT SOURCE FILE
#
# PROGRAM NAME :       aixcpu.sh
# DESCRIPTION :        This script has been developed to monitor and report the
#                      UNIX components. This script builds on an AIX script
#												written by a Wily client.
#
###############################################################################
#
###############################################################################
# CHANGE DESCRIPTION :    Draft version with FIX option
# 27/09/2007 - Sufyaan Kazi (CA Wily) - Tidied up script
#
#############################################################################################

#Function to write a metric in XML format for the EPA
# Arg 1 - Metric Name and Value (e.g. User=40)
# Arg 2 - Sub-Folder to stitch the metric in the Investigator Tree under HealthCheck
# Arg 3 - Metric Type, e.g. LongCounter
write_metric()
{
#.. PerIntervalCounter
#.. IntCounter
#.. IntAverage
#.. IntRate
#.. LongCounter
#.. LongAverage
	if [ "$#" == "5" ]; then
		print "<metric type=\"${1}\"  name=\"${2}|${3}:${4}\" value=\"${5}\"/>"
	else
		print "<metric type=\"${1}\"  name=\"${2}:${3}\" value=\"${4}\"/>"
	fi
}

WLMSTAT()
{
ptyp=
typ=class

wlmstat -av | egrep -v '(CLASS|TOTAL)' | while read cls tr ii prcs cpu x1 min smx hmx des rap urap pri mem m1 m2 m3 m4 m5 m6 m7 npg dsk rest
do

  if [ ! -z "$cls" ];then

    case $cls in
      *\.*) typ=subclass;;
      *) typ=class;;
    esac

    cpu=`echo $cpu | cut -f 1 -d"."`
    mem=`echo $mem | cut -f 1 -d"."`
    dsk=`echo $dsk | cut -f 1 -d"."`
    prcs=`echo $prcs | cut -f 1 -d"."`
    des=`echo $des | cut -f 1 -d"."`
    urap=`echo $urap | cut -f 1 -d"."`
    npg=`echo $npg | cut -f 1 -d"."`


    if [[ $typ == "subclass" ]]; then
     if [[ $cls != *\.Shared ]]; then     
      write_metric IntAverage AIX Wlm\|subclass\|$cls cpu $cpu
      write_metric IntAverage AIX Wlm\|subclass\|$cls mem $mem
      write_metric IntAverage AIX Wlm\|subclass\|$cls dsk $dsk
      write_metric IntAverage AIX Wlm\|subclass\|$cls prcs $prcs
      write_metric IntAverage AIX Wlm\|subclass\|$cls des $des
      write_metric IntAverage AIX Wlm\|subclass\|$cls urap $urap
      write_metric IntAverage AIX Wlm\|subclass\|$cls npg $npg
      #echo == subclass $pcls $pcpu $pmem $pdsk
     fi
    else
     if [[ $cls != Shared ]]; then
      #echo == $typ $cls $cpu $mem $dsk
      write_metric IntAverage AIX Wlm\|$typ\|$cls cpu $cpu
      write_metric IntAverage AIX Wlm\|$typ\|$cls dsk $dsk
      write_metric IntAverage AIX Wlm\|$typ\|$cls prcs $prcs
      write_metric IntAverage AIX Wlm\|$typ\|$cls urap $urap
      write_metric IntAverage AIX Wlm\|$typ\|$cls des $des
      write_metric IntAverage AIX Wlm\|$typ\|$cls npg $npg
      write_metric IntAverage AIX Wlm\|$typ\|$cls mem $mem
     else
      write_metric IntAverage AIX Wlm\|$typ\|$cls npg $npg
      write_metric IntAverage AIX Wlm\|$typ\|$cls mem $mem
     fi
    fi

  fi

done

}

#Function to count the no. of processes and zombies
NUM_procs()
{
        NUM_PROCS=`ps -efa | wc -l`
        write_metric IntCounter AIX Kernel Procs $NUM_PROCS
}


#Function to get LPARStat Metrics - subpool cpus
LPARSTAT()
{
    PCPUS=`lparstat | grep System | awk ' { print substr($8,7) } '`
    PCPUS=$(($PCPUS * 100))
    MAPP=$PCPUS
    CNT=3
    lparstat 1 $CNT | tail -$CNT | while read USR SYS WAIT IDLE PHYS ENT LBUSY APP VCWS PHINT; do
          #echo $APP
          APP=${APP%\.*}${APP#*\.}
          if [ "$APP" -lt "$MAPP" ]; then MAPP=$APP; fi
          TAPP=$(($TAPP + $APP)) 
    done

    APP=$(($TAPP/$CNT))
    APP=$(($PCPUS - $APP)) 
    MAPP=$(($PCPUS - $MAPP)) 
    APPP=$(($APP*100/$PCPUS)) 

    write_metric IntAverage AIX Cpu poolCpuPer $APPP
    write_metric IntAverage AIX Cpu poolCpuAvg $APP
    write_metric IntAverage AIX Cpu poolCpuMax $MAPP

}

#Function to get VMStat Metrics
VMSTAT()
{
        totcpu=`vmstat | grep lcpu | cut -d'=' -f2 | cut -d' ' -f1`
        vmstat | awk -f /usr/local/mware/iscope/scripts/AIX/vmstat1.awk
        CNT=3

        vmstat 1 $CNT | tail -$CNT | while read RUNQ WAITQ AVM FRE RE PI PO FR SR CY IN SI CS US SY ID WA PC EC; do
          #echo $RUNQ $WAITQ $AVM $FRE $RE $PI $PO $FR $SR $CY $IN $SI $CS $US $SY $ID $WA $PC $EC
          TRUNQ=$(($TRUNQ + $RUNQ)) 
          TWAITQ=$(($TWAITQ + $WAITQ)) 
          TIN=$(($TIN + $IN)) 
          TSI=$(($TSI + $SI)) 
          TCS=$(($TCS + $CS)) 
          TUS=$(($TUS + $US)) 
          TSY=$(($TSY + $SY)) 
          if [ "$chkmode" != "WPAR" ];then
             TID=$(($TID + $ID)) 
             TWA=$(($TWA + $WA)) 
          fi
          PC=${PC:0:1}${PC:2:2}
          EC=${EC%\.*}
          TPC=$(($TPC + $PC)) 
          TEC=$(($TEC + $EC)) 

          TAVM=$(($TAVM + $AVM)) 
          TFRE=$(($TFRE + $FRE)) 
          TRE=$(($TRE + $RE)) 
          TPI=$(($TPI + $PI)) 
          TPO=$(($TPO + $PO)) 
          TFR=$(($TFR + $FR)) 
          TSR=$(($TSR + $SR)) 
          TCY=$(($TCY + $CY)) 
        done

        RUNQ=$(($TRUNQ/$CNT))
        WAITQ=$(($TWAITQ/$CNT))
        IN=$(($TIN/$CNT))
        SI=$(($TSI/$CNT))
        CS=$(($TCS/$CNT))
        US=$(($TUS/$CNT))
        SY=$(($TSY/$CNT))
        if [ "$chkmode" != "WPAR" ];then
           ID=$(($TID/$CNT))
           WA=$(($TWA/$CNT))
        fi
        PC=$(($TPC/$CNT))
        PCP=$(($PC/$totcpu*2))
        EC=$(($TEC/$CNT))

        AVM=$(($TAVM/$CNT))
        FRE=$(($TFRE/$CNT))
        RE=$(($TRE/$CNT))
        PI=$(($TPI/$CNT))
        PO=$(($TPO/$CNT))
        FR=$(($TFR/$CNT))
        SR=$(($TSR/$CNT))
        CY=$(($TCY/$CNT))

        write_metric IntAverage AIX Kernel runQ $RUNQ
        write_metric IntAverage AIX Kernel waitQ $WAITQ
        write_metric IntAverage AIX Kernel deviceInterupts ${IN}
        write_metric IntAverage AIX Kernel systemInterupts ${SI}
        write_metric IntAverage AIX Kernel contextSwitches ${CS}
        write_metric IntAverage AIX Cpu user $US
        write_metric IntAverage AIX Cpu system $SY
        if [ "$chkmode" != "WPAR" ];then
           write_metric IntAverage AIX Cpu idle $ID
           write_metric IntAverage AIX Cpu wait $WA
        fi
        write_metric IntAverage AIX Cpu cpusUsed $PC
        write_metric IntAverage AIX Cpu cpusPercUsed $PCP
        write_metric IntAverage AIX Cpu entitlement $EC
        #write_metric IntCounter AIX Settings cpusavailable $totcpu

        write_metric LongCounter AIX Memory avm $AVM
        AVM=$(($AVM*4/1024))
        write_metric LongCounter AIX Memory average_used_MB $AVM
        write_metric LongCounter AIX Memory free $FRE
        FRE=$(($FRE*4/1024))
        write_metric LongCounter AIX Memory free_memory_MB $FRE
        write_metric IntAverage AIX Paging io $RE
        write_metric IntAverage AIX Paging in $PI
        write_metric IntAverage AIX Paging out $PO
        write_metric IntAverage AIX Paging freed $FR
        write_metric IntAverage AIX Paging scanned $SR
        write_metric IntAverage AIX Paging cycles $CY
}

#Function to get MPStat Metrics
MPSTAT()
{
  #set -x
  #totcpu=`vmstat | grep lcpu | cut -d'=' -f2 | cut -d' ' -f1`

  CNT=3
  CURCPU=x
  CUSED=0
  TPC=0
  TMAJ=0

  # count MAJ, PC, US, SY, WT, ID
  mpstat 1 $CNT                     | \
		grep -v -e '\-\-'                 \
		        -e 'config'               \
		        -e '^[ 	]*U'              \
						-e '^[ 	]*$'              \
						-e 'cpu.*min.*maj.*mpc' | \
		sort                            | \
	  while read CPU MIN MAJ MPC INT CS ICS RQ MIG LPA SYSC US SY WT ID PC EC LCS; do
                  #echo $CPU $MIN $MAJ $MPC $INT $CS $ICS $RQ $MIG $LPA $SYSC $US $SY $WT $ID $PC $EC $LCS

 			if [ "$CURCPU" == "x" ]; then
				CURCPU=$CPU
				CNT=0
				USED=0
			fi

			# This branch is executed when the current line describes a new CPU. (i.e. display the summary of the previous CPU)
			if [ "$CURCPU" != "$CPU" ]; then
				#echo new cpu $CURCPU $CNT $TMAJ $TPC
			# Divide values to get averages
				APC=$(($TPC/$CNT))
				AMAJ=$(($TMAJ/$CNT))

			# Output Metrics
				write_metric IntAverage CPU "Processor $CURCPU" "Utilization % (aggregate)" $APC
# 				#write_metric IntAverage CPU ${CURCPU}_Cpu $APC
				write_metric IntAverage CPU "Processor ${CURCPU}" "PageFaults" $AMAJ
								
				# If the LPA for this CPU was 100 for ANY of the Iterations (therefore excluding the U line, then add 1 to CUSED)
				if [ "$USED" == "1" -a "$CURCPU" != "U" ]; then
					CUSED=$(($CUSED+1))
				fi
				
				CURCPU=$CPU             # Update the current CPU 
				CNT=0                   # Reset CNT,TPC,TMAJ and USED
				TPC=0                   #   for the next CPU and iterations
				TMAJ=0                  #
				USED=0                  #
			fi
								
		# Major page faults (page faults with disk IO)
			if [ "$MAJ" != "-" ]; then
				TMAJ=$(($TMAJ + $MAJ))
			fi
				
		# Logical processor affinity. The percentage of logical processor redispatches within the scheduling affinity domain 3
			if [ "$LPA" == "100" ]; then
				USED=1
			fi
								
		# Fraction of physical processor consumed, displayed only in shared partition or when simultaneous multi-threading is enabled.
		# The pc of the cpuid U row represents the number of unused physical processors
			PC=${PC:0:1}${PC:2:2}    # Removes the decimal point (effectivly multiplies by 100)
			TPC=$(($TPC + $PC))
                        #echo $PC $TPC $CNT
						
			CNT=$(($CNT + 1))
		done						
	#echo end $CURCPU $TMAJ $TPC
	APC=$(($TPC/$CNT))
	AMAJ=$(($TMAJ/$CNT))
	write_metric IntAverage CPU TotalCpu $APC
	write_metric IntAverage CPU TotalPgFaults $AMAJ
	write_metric IntAverage CPU NumCpusUsed $CUSED
}				

chkmode=$1

WLMSTAT
NUM_procs
VMSTAT
MPSTAT
if [ "$chkmode" != "WPAR" ];then
   LPARSTAT
fi
