#!/bin/ksh93
#
###############################################################################
# SHELL SCRIPT SOURCE FILE
#
# PROGRAM NAME :       HealthCheck.sh
# DESCRIPTION :        This script has been developed to monitor and report the
#                      UNIX components. This script builds on an AIX script
#
###############################################################################
#
###############################################################################
# CHANGE DESCRIPTION :    Draft version with FIX option
# 27/09/2007 - Sufyaan Kazi (CA Wily) - Tidied up script
#
#############################################################################################


# Global variables

HOSTNAME=`hostname`
DATE=`date +%d-%m-%Y:%H:%m`

#Function to write a metric in XML format for the EPA
# Arg 1 - Metric Name and Value (e.g. User=40)
# Arg 2 - Sub-Folder to stitch the metric in the Investigator Tree under HealthCheck
# Arg 3 - Metric Type, e.g. LongCounter
write_metric()
{
        print "<metric type=\"${1}\"  name=\"AIX|${2}:${3}\" value=\"${4}\"/>"
}

#Function to get status of Network ports
UPTIME_status()
{
        uptime | read TIME x1 days x2 hours users x3 x4 x5 l1 l2 l3
        l1=${l1%\,*}
        l1=${l1%\.*}${l1#*\.}
        l2=${l2%\,*}
        l2=${l2%\.*}${l2#*\.}
        l3=${l3%\.*}${l3#*\.}
        #echo $TIME $days $hours $users $l1 $l2 $l3

        write_metric IntCounter Availability daysup $days
        write_metric IntCounter Availability users $users
        write_metric IntAverage Cpu load1 $l1
        write_metric IntAverage Cpu load2 $l2
        write_metric IntAverage Cpu load3 $l3
}

#Function to get status of sockets
NETSTAT_status()
{
        perl /usr/local/mware/iscope/scripts/AIX/netstatsumm.pl
        /usr/local/mware/iscope/scripts/AIX/netstats.ksh
}


#Function to get status of sockets
NFSSTAT_status()
{
        /usr/local/mware/iscope/scripts/AIX/nfsstats.ksh
}


#Function to get status of Network ports
ETHERNET_status()
{
        for i in `netstat -i | grep -i en | cut -d" " -f1 | sort | uniq`
        do
                typeset -i DEV_I=1
                DEV=`entstat -d $i | grep -i "Link Status" | cut -d" " -f3`
                if [[ $DEV != "Up" ]]
                then
                        DEV_I=0
                fi

                write_metric IntCounter Network\|${i} status $DEV_I
        done
}

#Function to get size of packets and errors
NETWORK_loads()
{
        #set -x
        for NI in `netstat -i | grep -i en | cut -d" " -f1 | sort | uniq`
        do
                NIC=`netstat -i | grep -i ${NI} | grep -iv link | awk '{ printf ("%s,%s,%s,%s,%s,%s,%s,%s,%s\n", $1,$2,$3,$4,$5,$6,$7,$8,$9) }'`
                if [ "$chkmode" != "WPAR" ];then
                   IPkts=`print $NIC | cut -f5 -d "," `
                   Ierrs=`print $NIC | cut -f6 -d "," `
                   OPkts=`print $NIC | cut -f7 -d "," `
                   Oerrs=`print $NIC | cut -f8 -d "," `
                else
                   IPkts=`print $NIC | cut -f6 -d "," `
                   Ierrs=`print $NIC | cut -f7 -d "," `
                   OPkts=`print $NIC | cut -f8 -d "," `
                   Oerrs=`print $NIC | cut -f9 -d "," `
                fi
                if [ -e /tmp/statsnic_${USER}_${NI} ];then
                  cat /tmp/statsnic_${USER}_${NI} | read xIPkts xIerrs xOPkts xOerrs
                  #echo $xIPkts $xIerrs $xOPkts $xOerrs
                  ip=$(($IPkts-${xIPkts:-0}))
                  ie=$(($Ierrs-${xIerrs:-0}))
                  op=$(($OPkts-${xOPkts:-0}))
                  oe=$(($Oerrs-${xOerrs:-0}))
                
                  write_metric PerIntervalCounter Network\|${NI} In_Packets ${ip}
                  write_metric PerIntervalCounter Network\|${NI} In_Errors ${ie}
                  write_metric PerIntervalCounter Network\|${NI} Out_Packets ${op}
                  write_metric PerIntervalCounter Network\|${NI} Out_Errors ${oe}
                fi
                echo $IPkts $Ierrs $OPkts $Oerrs > /tmp/statsnic_${USER}_${NI}

        done
}

#Function to get IO on disks
DRIVEIO_loads()
{
   typeset -A hdvg
   typeset -A vgmps

   hacmp=0
   if which lsvpcfg >/dev/null 2>&1
   then
     lsvpcfg | while read vpath x1 x2 vg x3 x4 hd1 x5 x6 hd2 x7 x8
     do
       hacmp=1
       l=${#vg}
       l=$((l-1))
       vg=${vg:0:$l}
       hdvg[$hd1]=$vg
       hdvg[$hd2]=$vg
     done
   fi


   lsvg -o | while read vg
   do
      lsvg -l $vg | tail +3 | while read lv ftyp lps pps pvs lvstate mp rest
      do
        if [ ! "$mp" == "N/A" ];then
          vgmps[$vg]=${vgmps[$vg]},$mp
        fi
      done
      write_metric StringEvent Disk\|${vg} mounts "${vgmps[$vg]}"

      if [ $hacmp == 0 ];then
        lsvg -p $vg | tail +3 | while read hd rest
        do
          hdvg[$hd]=$vg
        done
      fi

   done


        set -A dact
        set -A dkps
        set -A dtps
        dact=([hdisk0]=0)
        dkps=([hdisk0]=0)
        dtps=([hdisk0]=0)
        iostat 1 5 | grep hdisk | while read disk act kps tps kbr kbw
        do
           #echo $disk $act $kps $tps $kbr $kbw
           #echo $disk ${dact[$disk]} ${dkps[$disk]} ${dtps[$disk]} 
           dact[$disk]=$((${dact[$disk]}+$act))
           dkps[$disk]=$((${dkps[$disk]}+$kps))
           dtps[$disk]=$((${dtps[$disk]}+$tps))
           #echo $disk ${dact[$disk]} ${dkps[$disk]} ${dtps[$disk]} 
        done

        iostat | tail +8 | while read disk a b c kbr kbw
        do
                vg=${hdvg[$disk]}

                a=${dact[$disk]/\.*}
                k=${dkps[$disk]/\.*}
                t=${dtps[$disk]/\.*}
                act=$((${a}/5)) 
                kps=$((${k}/5)) 
                tps=$((${t}/5)) 
                write_metric IntAverage Disk\|${vg}\|${disk} active ${act}
                write_metric IntAverage Disk\|${vg}\|${disk} kps ${kps}
                write_metric IntAverage Disk\|${vg}\|${disk} tps ${tps}

                if [ -e /tmp/statsdisk_${USER}_${disk} ];then
                  cat /tmp/statsdisk_${USER}_${disk} | read xkbr xkbw
                  
                  if [ ! -z "$xkbr" ]; then
                    if [ ! -z "$xkbw" ]; then
                  
                       skbr=$(($kbr-$xkbr))
                       skbw=$(($kbw-$xkbw))
                
                       write_metric PerIntervalCounter Disk\|${vg}\|${disk} kbRead ${skbr}
                       write_metric PerIntervalCounter Disk\|${vg}\|${disk} kbWritten ${skbw}
                    fi
                  fi
                fi
                echo $kbr $kbw > /tmp/statsdisk_${USER}_${disk}
        done

}

# Main script

. /usr/local/mware/bin/ksh.function_library

dontRunMeTwice
trap exitTrap 1 2 3 6 9 14 15

chkmode=$1

UPTIME_status
NETWORK_loads
if [ "$chkmode" != "WPAR" ];then
  ETHERNET_status
  DRIVEIO_loads
fi
NETSTAT_status
NFSSTAT_status

dontRunMeTwice rm
