#! perl -w
###############################################################################
#
# pingcheck.pl
#
###############################################################################
#
# A script to ping hosts
#
###############################################################################
#
# History
# -------
#
# 20100522 - Warren - Created
# yyyymmdd - who@where - Comments...
#
###############################################################################

###############################################################################
# Import modules
###############################################################################
use strict;
use Net::Ping;
use Getopt::Long;
use IO::Socket;
use IO::Handle;
use Time::HiRes;


my $hostlist_filename = '';
my $remote_epaagent = '';
my $host = '';
my $url_EPAgent = "";
my $url_to_EPAgent = "";
my $connection_result = "";
my $result = "";

###############################################################################
# Functions
###############################################################################
sub WriteMetricToEPagent {

	#############################################################################
	# Declare variables
	#############################################################################
	my $remote_url = shift;
	my $remote_protocol = "tcp";
	my $remote_timeout = 1;
	my $EOL = "\015\012";
	my $BLANK = $EOL x 2;
	my $url_request_result;

	#############################################################################
	# Transform the remote URL variable
	#############################################################################
	next if $remote_url =~ /^\s*$/; # in case of custom delims, may have empty elems
	$remote_url =~ s/^\s*//; # trim preceding and
	$remote_url =~ s/\s*$//; # trailing whitespace

	my ($protocol, $remote_host, $remote_port, $remote_document) = $remote_url =~ m@^(https?)://([\w.\-]+):?(\d+)?(.*)@i;
	if ( !defined($remote_port) ) {
		$remote_port = "80";
	}


	#############################################################################
	# Make connection
	#############################################################################
	my $remote_connection = IO::Socket::INET->new(
		Proto => $remote_protocol,
		PeerAddr => $remote_host,
		Timeout => $remote_timeout,
		PeerPort  => $remote_port
	);

	#############################################################################
	# Request the document if the connection is made
	#############################################################################
	if ($remote_connection) {
		$remote_connection->autoflush(1);
		print $remote_connection "GET $remote_document HTTP/1.1\nUser-Agent: Introscope\nHost: $remote_host:$remote_port" . $BLANK;
		$url_request_result = <$remote_connection>;
		close $remote_connection;
	}
	else { 
		$url_request_result =  "NO_CONNECTION"
	}

	return $url_request_result;
}


###############################################################################
# MAIN
###############################################################################
# Interrogate script arguments
###############################################################################
GetOptions (
	'file=s' => \$hostlist_filename,
	'epagent=s' => \$remote_epaagent
);

if (( $hostlist_filename eq '' ) || ( $remote_epaagent eq '' )) 
{
    die "usage: \'$0 -file <filename> -epaagent <target epagent>\'"; 
}

$url_EPAgent = "http://" . $remote_epaagent;

###############################################################################
# Open the file and read list into array
###############################################################################
chomp($hostlist_filename);
print "Reading host list from $hostlist_filename";

#$hostlist_filename='hostlist.txt';
open(DAT, $hostlist_filename) || die("Could not open file!");
my @hostlist=<DAT>;
close(DAT);



###############################################################################
# Iterate through the array and check hosts
###############################################################################
my $p = Net::Ping->new();

foreach $host (@hostlist)
{
    chomp($host);
    
    my $start_time = [ Time::HiRes::gettimeofday( ) ];
    if ( $p->ping($host) )
    {
       $url_to_EPAgent = $url_EPAgent . "/?metricType=IntCounter&metricName=PING|" . $host . ":up&metricValue=1";
       $result = WriteMetricToEPagent ($url_to_EPAgent);
       print "PING|$host:up=1\n";
    }
    else
    {
       $url_to_EPAgent = $url_EPAgent . "/?metricType=IntCounter&metricName=PING|" . $host . ":up&metricValue=0";
       $result = WriteMetricToEPagent ($url_to_EPAgent);
       print "PING|$host:up=0\n";
    }
    my $resptime = int(Time::HiRes::tv_interval( $start_time ) * 1000);

    $url_to_EPAgent = $url_EPAgent . "/?metricType=IntAverage&metricName=PING|" . $host . ":respTime&metricValue=" . $resptime;
    $result = WriteMetricToEPagent ($url_to_EPAgent);
    print "PING|$host:resptime=$resptime\n";

}

$p->close();






  #
  #  Other ways to do ping - FYIO
  #
  #print "$host is alive.\n" if pingecho($host);
  #  $p = Net::Ping->new("icmp");
  #  foreach $host (@host_array)
  #  {
  #      print "$host is ";
  #      print "NOT " unless $p->ping($host, 2);
  #      print "reachable.\n";
  #      sleep(1);
  #  }
  #  $p->close();
  #  
  #  $p = Net::Ping->new("tcp", 2);
  #  while ($stop_time > time())
  #  {
  #      print "$host not reachable ", scalar(localtime()), "\n"
  #          unless $p->ping($host);
  #      sleep(300);
  #  }
  #  undef($p);
  #  
  #  # For backward compatibility
  #  print "$host is alive.\n" if pingecho($host);

